part of tags;

final Tag toggleButton = createTag((BindModel onBind) => 
    bindModel(new ToggleButton(), onBind));

/// A button that allows toggling between an [isUp] and an [isDown] state.
class ToggleButton implements HasElement {
  
  final Element element = 
      button(() {
        shadow(() {
          content()..select='.up';  
        });
      });
  
  ContentElement _content;  
  
  bool _isDown = false;
  /// Whether or not this button is down.
  bool get isDown => _isDown;
       set isDown(bool value) {
         if (value != _isDown) {
           _isDown = value;           
           if (_isDown) _content.select = '.down';
           else _content.select = '.up';
           if (_states.hasListener && !_states.isPaused) _states.add(_isDown);
         }
       }
  
  /// Whether or not this button is up.
  bool get isUp => !_isDown;
  
  final _states = new StreamController<bool>();
  /// An observable stream of this button's state transitions.
  /// A _true_ value represents [isDown] and a _false_ value represents [isUp].
  Stream<bool> get states => _states.stream;
  
  /// Constructs a new toggle button.  
  ToggleButton() {
    element.onClick.listen((_) => isDown = !_isDown);
    _content = element.shadowRoot.children[0];
  }
}
